# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-10 10:45
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="AdyenIssuer",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(default="", max_length=200)),
                ("adyen_id", models.CharField(default="", max_length=200)),
            ],
        ),
        migrations.CreateModel(
            name="AdyenNotification",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("notification", models.TextField()),
                ("is_processed", models.BooleanField(default=False)),
                ("processed_at", models.DateTimeField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name="AdyenPaymentOption",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(default="", max_length=200)),
                ("adyen_name", models.CharField(default="", max_length=200)),
                (
                    "guid",
                    models.CharField(default="", max_length=36, verbose_name="GUID"),
                ),
                ("image", models.ImageField(null=True, upload_to="")),
                ("is_active", models.BooleanField(default=False)),
            ],
        ),
        migrations.AddField(
            model_name="adyenissuer",
            name="payment_option",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="djadyen.AdyenPaymentOption",
            ),
        ),
    ]



from django.utils.translation import gettext_lazy as _

from djchoices import ChoiceItem, DjangoChoices


class Status(DjangoChoices):
    Authorised = ChoiceItem("authorised", _("Authorised"))
    Cancel = ChoiceItem("cancel", _("Cancel"))
    Created = ChoiceItem("created", _("Created"))
    Error = ChoiceItem("error", _("Error"))
    Pending = ChoiceItem("pending", _("Pending"))
    Refused = ChoiceItem("refused", _("Refused"))



import json
import logging

from django.http import HttpResponse
from django.utils.decorators import method_decorator
from django.utils.translation import gettext_lazy as _
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import View

from ..models import AdyenNotification
from .hmac import get_signature

logger = logging.getLogger(__name__)


class NotificationView(View):
    @method_decorator(csrf_exempt)
    def dispatch(self, request, *args, **kwargs):
        return super(NotificationView, self).dispatch(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        """ """
        logger.debug(_("New notification(s)"))
        json_params = json.loads(request.body)
        notification_items = json_params.get("notificationItems", [])
        successfully_created = True
        for notification_item in notification_items:
            nir = notification_item.get("NotificationRequestItem")
            signature = nir.get("additionalData", {}).get("hmacSignature")
            create_notification = False
            if signature:
                compare_signature = get_signature(nir)
                if signature == compare_signature:
                    create_notification = True

            if create_notification:
                notification = AdyenNotification.objects.create(
                    notification=json.dumps(nir)
                )
                logger.debug(_("Notification saved | id: %s"), notification.id)
            else:
                successfully_created = False

        if successfully_created:
            return HttpResponse("[accepted]")
        return HttpResponse("[rejected]")



import json
import logging
from uuid import uuid4

from django.db import models
from django.utils import timezone
from django.utils.translation import gettext_lazy as _

from djadyen import settings

from .choices import Status

logger = logging.getLogger(__name__)


class AdyenNotification(models.Model):
    created_at = models.DateTimeField(auto_now_add=True)
    notification = models.TextField()
    is_processed = models.BooleanField(default=False)
    processed_at = models.DateTimeField(blank=True, null=True)

    def __str__(self):
        return self.created_at.strftime("%c")

    def get_notification_data(self):
        return json.loads(self.notification)

    def has_status(self, status):
        data = self.get_notification_data()
        event_code = data.get("eventCode")
        merchant_account_code = data.get("merchantAccountCode")
        return (
            event_code == status
            and merchant_account_code == settings.DJADYEN_MERCHANT_ACCOUNT
        )

    def is_authorised(self, require_success=True):
        data = self.get_notification_data()
        success = data.get("success") == "true"
        has_status = self.has_status("AUTHORISATION")

        if not require_success:
            return has_status

        return has_status and success

    def is_error(self):
        data = self.get_notification_data()
        return self.has_status("ERROR") or data.get("success") == "false"

    def is_cancelled(self):
        return self.has_status("CANCEL")

    def is_refused(self):
        return self.has_status("REFUSED")

    def mark_processed(self, commit=True):
        self.is_processed = True
        self.processed_at = timezone.now()
        if commit:
            self.save()


class AdyenPaymentOption(models.Model):
    name = models.CharField(max_length=200, default="")
    adyen_name = models.CharField(max_length=200, default="")
    guid = models.CharField(max_length=36, verbose_name=_("GUID"), default="")
    image = models.ImageField(null=True)
    is_active = models.BooleanField(default=False)

    def __str__(self):
        return self.name


class AdyenIssuer(models.Model):
    payment_option = models.ForeignKey(AdyenPaymentOption, on_delete=models.CASCADE)
    name = models.CharField(max_length=200, default="")
    adyen_id = models.CharField(max_length=200, default="")

    def __str__(self):
        return self.name


class AdyenOrder(models.Model):
    status = models.CharField(
        max_length=200, choices=Status.choices, default=Status.Created
    )
    created_on = models.DateTimeField(auto_now_add=True)
    reference = models.CharField(max_length=200, default="", blank=True)
    psp_reference = models.CharField(max_length=200, default="", blank=True)
    email = models.EmailField()

    payment_option = models.ForeignKey(
        AdyenPaymentOption, blank=True, null=True, on_delete=models.SET_NULL
    )
    issuer = models.ForeignKey(
        AdyenIssuer, null=True, blank=True, on_delete=models.SET_NULL
    )

    __old_status = None

    class Meta:
        abstract = True

    def __str__(self):
        return "{}".format(self.reference)

    def __init__(self, *args, **kwargs):
        super(AdyenOrder, self).__init__(*args, **kwargs)
        self.__old_status = self.status

    def get_return_url(self):
        raise NotImplementedError(
            "Please override 'get_return_url' on the '{model_name}'".format(
                model_name=self._meta.object_name
            )
        )

    def process_notification(self, notification):
        if notification.is_authorised():
            self.status = Status.Authorised
            self.save()
            notification.mark_processed()
        elif notification.is_cancelled():
            self.status = Status.Cancel
            self.save()
            notification.mark_processed()
        elif notification.is_refused():
            self.status = Status.Refused
            self.save()
            notification.mark_processed()
        elif notification.is_error():
            self.status = Status.Error
            self.save()
            notification.mark_processed()

        logger.error("Can't process notification with pk %d", notification.pk)

        # Ignore anything else for now.

    def get_price_in_cents(self):
        """
        :return int Return the price in cents for this order.
        """
        raise NotImplementedError

    def save(self, can_change=False, *args, **kwargs):
        if not self.reference:
            self.reference = uuid4()

        if settings.DJADYEN_REFETCH_OLD_STATUS and self.id:
            self.__old_status = self._meta.model.objects.get(pk=self.id).status

        if self.status != self.__old_status:
            if self.__old_status == Status.Authorised:
                logger.warning(
                    _(
                        "Order ref: %s | Tried to change the "
                        + "status from 'Authorised' to '%s'."
                    ),
                    self.reference,
                    self.status,
                )
                self.status = self.__old_status
            else:
                logger.warning(
                    _("Order ref: %s | Changed the status from '%s' to '%s'"),
                    self.reference,
                    self.__old_status,
                    self.status,
                )
                self.__old_status = self.status

        return super(AdyenOrder, self).save(*args, **kwargs)



from django.core.management.base import BaseCommand

import Adyen

from djadyen import settings

from ...models import AdyenIssuer, AdyenPaymentOption


def create_payment_method(type, name, issuers):
    payment_qs = AdyenPaymentOption.objects.filter(adyen_name=type)
    if not payment_qs.exists():
        payment = AdyenPaymentOption.objects.create(
            name=name,
            adyen_name=type,
        )
    else:
        payment = payment_qs.first()

    for issuer in issuers:
        issuer_qs = AdyenIssuer.objects.filter(adyen_id=issuer["id"])
        if issuer_qs.exists():
            issuer_obj = issuer_qs.first()
            issuer_obj.name = issuer.get("name")
            issuer_obj.payment_option = payment
            issuer_obj.save()
        else:
            issuer_obj = AdyenIssuer.objects.create(
                name=issuer.get("name"),
                adyen_id=issuer.get("id"),
                payment_option=payment,
            )


class Command(BaseCommand):
    help = "Sync the payment methods from adyen."

    def handle(self, *args, **options):
        ady = Adyen.Adyen()

        # Setting global values
        ady.payment.client.platform = settings.DJADYEN_ENVIRONMENT
        ady.payment.client.xapikey = settings.DJADYEN_SERVER_KEY
        ady.payment.client.app_name = settings.DJADYEN_APPNAME

        # Setting request data.
        request = {
            "merchantAccount": settings.DJADYEN_MERCHANT_ACCOUNT,
        }
        # Starting the checkout.
        result = ady.checkout.payment_methods(request)

        payment_methods = result.message.get("paymentMethods")
        for payment_method in payment_methods:
            name = payment_method.get("name")
            type = payment_method.get("type")
            brands = payment_method.get("brands")
            issuers = payment_method.get("issuers", [])
            if brands:
                for brand in brands:
                    create_payment_method(brand, name, issuers)
            else:
                create_payment_method(type, name, issuers)



from django.contrib import admin

from .models import AdyenIssuer, AdyenNotification, AdyenPaymentOption


@admin.register(AdyenNotification)
class AdyenNotificationAdmin(admin.ModelAdmin):
    list_display = ("created_at", "is_processed", "processed_at")
    list_filter = ("is_processed", "created_at", "processed_at")


@admin.register(AdyenPaymentOption)
class PaymentOptionAdmin(admin.ModelAdmin):
    list_display = ("name", "adyen_name", "is_active")
    list_filter = ("is_active",)


@admin.register(AdyenIssuer)
class IssuerAdmin(admin.ModelAdmin):
    list_display = ("name", "payment_option")
    list_filter = ("payment_option",)



import logging
from django import template
from django.utils.translation import get_language

import Adyen

from djadyen import settings
from djadyen.choices import Status

register = template.Library()
logger = logging.getLogger(__name__)


@register.inclusion_tag("adyen/component.html")
def adyen_payment_component(
    language, order, merchant_account=settings.DJADYEN_MERCHANT_ACCOUNT
):
    """
    Will display a singular payment method.
    """
    ady = Adyen.Adyen()

    # Setting global values
    ady.payment.client.platform = settings.DJADYEN_ENVIRONMENT
    ady.payment.client.xapikey = settings.DJADYEN_SERVER_KEY
    ady.payment.client.app_name = settings.DJADYEN_APPNAME
    # Setting request data.
    request = {
        "amount": {
            "value": order.get_price_in_cents(),
            "currency": settings.DJADYEN_CURRENCYCODE,
        },
        "reference": str(order.reference),
        "merchantAccount": merchant_account,
        "returnUrl": order.get_return_url(),
        "shopperLocale": language,
        "countryCode": language,
    }
    logger.exception(request)
    # Starting the checkout.
    result = ady.checkout.sessions(request)

    if result.status_code == 201:
        return {
            "client_key": settings.DJADYEN_CLIENT_KEY,
            "session_id": result.message.get("id"),
            "session_data": result.message.get("sessionData"),
            "environment": settings.DJADYEN_ENVIRONMENT,
            "redirect_url": order.get_return_url,
            "language": get_language(),
            "payment_type": order.payment_option.adyen_name
            if order.payment_option
            else "",
            "issuer": order.issuer.adyen_id if order.issuer else "",
        }
    return {}


@register.inclusion_tag("adyen/polling.html")
def adyen_status_polling(order, status_url):
    return {
        "status_url": status_url,
        "pending": order.status in [Status.Created, Status.Pending],
    }



import logging

from django.http import JsonResponse, HttpResponseRedirect
from django.views.generic.detail import DetailView

from .choices import Status

logger = logging.getLogger("adyen")


class AdyenPaymentView(DetailView):
    """
    A view which initiates the Adyen payment by rendering a widget.
    It will automatically draw the wanted widget.
    """

    template_name = "adyen/pay.html"
    slug_field = "reference"
    slug_url_kwarg = "reference"

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        if self.object.get_price_in_cents() == 0:
            return HttpResponseRedirect(self.object.get_return_url())
        return super(AdyenPaymentView, self).get(request, *args, **kwargs)


class AdyenResponseView(DetailView):
    slug_field = "reference"
    slug_url_kwarg = "reference"

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        if self.object.get_price_in_cents() == 0:
            self.handle_authorised(self.object)
        return super(AdyenResponseView, self).get(request, *args, **kwargs)

    def handle_authorised(self, order):
        raise NotImplementedError()


class AdyenOrderStatusView(DetailView):
    slug_field = "reference"
    slug_url_kwarg = "reference"

    def get(self, request, *args, **kwargs):
        order = self.get_object()
        updated_status = True
        if order.status in [Status.Created, Status.Pending]:
            updated_status = False

        return JsonResponse(
            {
                "updatedStatus": updated_status,
                "currentStatus": order.status,
                "reference": order.reference,
            }
        )



try:
    from django.urls import path
except Exception:
    from django.conf.urls import url as path

from .views import NotificationView

app_name = "adyen-notifications"
urlpatterns = [
    path("", NotificationView.as_view(), name="notification"),
]



from django.conf import settings as dj_settings

DJADYEN_SERVER_KEY = getattr(dj_settings, "DJADYEN_SERVER_KEY")
DJADYEN_CLIENT_KEY = getattr(dj_settings, "DJADYEN_CLIENT_KEY")
DJADYEN_ENVIRONMENT = getattr(dj_settings, "DJADYEN_ENVIRONMENT", "test")
DJADYEN_APPNAME = getattr(dj_settings, "DJADYEN_APPNAME", "Djadyen Payment")
DJADYEN_MERCHANT_ACCOUNT = getattr(dj_settings, "DJADYEN_MERCHANT_ACCOUNT")
DJADYEN_CURRENCYCODE = getattr(dj_settings, "DJADYEN_CURRENCYCODE", "EUR")
DJADYEN_REFETCH_OLD_STATUS = getattr(dj_settings, "DJADYEN_REFETCH_OLD_STATUS", False)
DJADYEN_HANDLE_NOTIFICATION_MINUTES_AGO = getattr(
    dj_settings, "DJADYEN_HANDLE_NOTIFICATION_MINUTES_AGO", 15
)
DJADYEN_ORDER_MODELS = getattr(dj_settings, "DJADYEN_ORDER_MODELS")
DJADYEN_NOTIFICATION_KEY = getattr(dj_settings, "DJADYEN_NOTIFICATION_KEY")
